@@ -1,5 +1,5 @@
 /*
- * $Id: hqx.c,v 1.8.2.1 2002/03/12 14:12:40 srittau Exp $
+ * $Id: hqx.c,v 1.12.4.1 2003/09/03 20:40:50 didg Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,7 @@
 #include <netatalk/endian.h>
 
 #include "megatron.h"
+#include "nad.h"
 #include "hqx.h"
 
 #define HEXOUTPUT	0
@@ -110,6 +111,7 @@ int hqx_open( hqxfile, flags, fh, options )
 #if DEBUG
     fprintf( stderr, "megatron: entering hqx_open\n" );
 #endif /* DEBUG */
+    select_charset( options);
     if ( flags == O_RDONLY ) {
 
 #if HEXOUTPUT
