@@ -1,5 +1,5 @@
 /*
- * $Id: appl.c,v 1.7.2.1 2003/06/09 15:09:06 srittau Exp $
+ * $Id: appl.c,v 1.12.2.1.2.2 2004/02/20 21:22:57 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -13,19 +13,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#include <sys/types.h>
-#include <sys/stat.h>
+
 #include <sys/param.h>
 #include <atalk/logger.h>
 #include <errno.h>
 
-#include <netatalk/endian.h>
 #include <atalk/adouble.h>
 #include <atalk/afp.h>
 
@@ -134,7 +126,8 @@ u_short	mplen;
  * but uses upaths instead of mac format paths.
  *
  * The new way: dir and path refer to an app, path is a mac format
- * pathname.  makemacpath() builds a cname.
+ * pathname.  makemacpath() builds a cname. (zero is a path separator
+ * and it's not \0 terminated).
  *
  * See afp_getappl() for the backward compatiblity code.
  */
@@ -149,11 +142,15 @@ char	*path;
 
     p = mpath + mpathlen;
     p -= strlen( path );
-    strncpy( p, path, strlen( path ));
+    memcpy( p, path, strlen( path )); 
 
     while ( dir->d_parent != NULL ) {
-        p -= strlen( dir->d_name ) + 1;
-        strcpy( p, dir->d_name );
+        p -= strlen( dir->d_m_name ) + 1;
+        if (p < mpath) {
+            /* FIXME: pathname too long */
+            return NULL;
+        }
+        strcpy( p, dir->d_m_name );
         dir = dir->d_parent;
     }
     return( p );
@@ -170,7 +167,8 @@ int		ibuflen, *rbuflen;
     int			tfd, cc;
     u_int32_t           did;
     u_int16_t		vid, mplen;
-    char		*path, *dtf, *p, *mp;
+    struct path         *path;
+    char                *dtf, *p, *mp;
     u_char		creator[ 4 ];
     u_char		appltag[ 4 ];
     char		*mpath, *tempfile;
@@ -187,7 +185,7 @@ int		ibuflen, *rbuflen;
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
     if (NULL == ( dir = dirlookup( vol, did )) ) {
-        return( AFPERR_NOOBJ );
+        return afp_errno;
     }
 
     memcpy( creator, ibuf, sizeof( creator ));
@@ -197,9 +195,9 @@ int		ibuflen, *rbuflen;
     ibuf += sizeof( appltag );
 
     if (NULL == ( path = cname( vol, dir, &ibuf )) ) {
-        return( AFPERR_NOOBJ );
+        return get_afp_errno(AFPERR_PARAM);
     }
-    if ( *path == '\0' ) {
+    if ( path_isadir(path) ) {
         return( AFPERR_BADTYPE );
     }
 
@@ -216,7 +214,10 @@ int		ibuflen, *rbuflen;
         return( AFPERR_PARAM );
     }
     mpath = obj->newtmp;
-    mp = makemacpath( mpath, AFPOBJ_TMPSIZ, curdir, path );
+    mp = makemacpath( mpath, AFPOBJ_TMPSIZ, curdir, path->m_name );
+    if (!mp) {
+        return AFPERR_PARAM;
+    }
     mplen =  mpath + AFPOBJ_TMPSIZ - mp;
 
     /* write the new appl entry at start of temporary file */
@@ -256,7 +257,8 @@ int		ibuflen, *rbuflen;
     int			tfd, cc;
     u_int32_t           did;
     u_int16_t		vid, mplen;
-    char		*path, *dtf, *mp;
+    struct path    	*path;
+    char                *dtf, *mp;
     u_char		creator[ 4 ];
     char                *tempfile, *mpath;
 
@@ -272,16 +274,16 @@ int		ibuflen, *rbuflen;
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
     if (NULL == ( dir = dirlookup( vol, did )) ) {
-        return( AFPERR_NOOBJ );
+        return afp_errno;
     }
 
     memcpy( creator, ibuf, sizeof( creator ));
     ibuf += sizeof( creator );
 
     if (NULL == ( path = cname( vol, dir, &ibuf )) ) {
-        return( AFPERR_NOOBJ );
+        return get_afp_errno(AFPERR_PARAM);
     }
-    if ( *path == '.' ) {
+    if ( path_isadir(path) ) {
         return( AFPERR_BADTYPE );
     }
 
@@ -298,7 +300,11 @@ int		ibuflen, *rbuflen;
         return( AFPERR_PARAM );
     }
     mpath = obj->newtmp;
-    mp = makemacpath( mpath, AFPOBJ_TMPSIZ, curdir, path );
+    mp = makemacpath( mpath, AFPOBJ_TMPSIZ, curdir, path->m_name );
+    if (!mp) {
+        return AFPERR_PARAM ;
+    }
+
     mplen =  mpath + AFPOBJ_TMPSIZ - mp;
     cc = copyapplfile( sa.sdt_fd, tfd, mp, mplen );
     close( tfd );
@@ -320,7 +326,6 @@ AFPObj      *obj;
 char	*ibuf, *rbuf;
 int		ibuflen, *rbuflen;
 {
-    struct stat		st;
     struct vol		*vol;
     char		*p, *q;
     int			cc, buflen;
@@ -328,7 +333,8 @@ int		ibuflen, *rbuflen;
     u_char		creator[ 4 ];
     u_char		appltag[ 4 ];
     char                *buf, *cbuf;
-
+    struct path         *path;
+    
     ibuf += 2;
 
     memcpy( &vid, ibuf, sizeof( vid ));
@@ -434,17 +440,17 @@ int		ibuflen, *rbuflen;
     memcpy( q, p, len );
     q = cbuf;
 
-    if (( p = cname( vol, vol->v_dir, &q )) == NULL ) {
+    if (( path = cname( vol, vol->v_dir, &q )) == NULL ) {
         *rbuflen = 0;
         return( AFPERR_NOITEM );
     }
 
-    if ( stat( mtoupath(vol, p), &st ) < 0 ) {
+    if ( path_isadir(path) || path->st_errno ) {
         *rbuflen = 0;
         return( AFPERR_NOITEM );
     }
     buflen = *rbuflen - sizeof( bitmap ) - sizeof( appltag );
-    if ( getfilparams(vol, bitmap, p, curdir, &st, rbuf + sizeof( bitmap ) +
+    if ( getfilparams(vol, bitmap, path, curdir, rbuf + sizeof( bitmap ) +
                       sizeof( appltag ), &buflen ) != AFP_OK ) {
         *rbuflen = 0;
         return( AFPERR_BITMAP );
