@@ -1,5 +1,5 @@
 /*
- * $Id: ofork.c,v 1.19.2.1 2003/06/09 14:53:16 srittau Exp $
+ * $Id: ofork.c,v 1.20.6.6.2.1 2004/12/07 03:23:51 didg Exp $
  *
  * Copyright (c) 1996 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -11,13 +11,16 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <string.h>
 #include <sys/stat.h> /* works around a bug */
 #include <sys/param.h>
 #include <atalk/logger.h>
 #include <errno.h>
 
-#include <atalk/adouble.h>
+#include <atalk/util.h>
 
 #include "globals.h"
 #include "volume.h"
@@ -66,6 +69,7 @@ static __inline__ void of_unhash(struct ofork *of)
     }
 }
 
+#ifdef DEBUG1
 void of_pforkdesc( f )
 FILE	*f;
 {
@@ -80,6 +84,7 @@ FILE	*f;
         }
     }
 }
+#endif
 
 int of_flush(const struct vol *vol)
 {
@@ -115,7 +120,7 @@ const char *oldpath, *newpath;
         if (vol == of->of_vol && olddir == of->of_dir &&
 	         s_of->key.dev == of->key.dev && 
 	         s_of->key.inode == of->key.inode ) {
-            strncpy( of->of_name, newpath, of->of_namelen);
+            strlcpy( of->of_name, newpath, of->of_namelen);
             if (newdir != olddir) {
                 of->of_d_prev->of_d_next = of->of_d_next;
                 of->of_d_next->of_d_prev = of->of_d_prev;
@@ -215,7 +220,7 @@ struct stat     *st;
 
         /* initialize to zero. This is important to ensure that
            ad_open really does reinitialize the structure. */
-        memset( ad, 0, sizeof( struct adouble ) );
+        ad_init(ad, vol->v_adouble);
     } else {
         /* Increase the refcount on this struct adouble. This is
            decremented again in oforc_dealloc. */
@@ -248,7 +253,7 @@ struct stat     *st;
         oforks[ of_refnum ] = NULL;
         return NULL;
     }
-    strncpy( of->of_name, path, of->of_namelen = 255 + 1);
+    strlcpy( of->of_name, path, of->of_namelen = 255 + 1);
     *ofrefnum = refnum;
     of->of_refnum = refnum;
     of->key.dev = st->st_dev;
@@ -270,22 +275,64 @@ struct ofork *of_find(const u_int16_t ofrefnum )
     return( oforks[ ofrefnum % nforks ] );
 }
 
-/* --------------------------
-*/
+/* -------------------------- */
+int of_stat  (struct path *path)
+{
+int ret;
+    path->st_errno = 0;
+    path->st_valid = 1;
+    if ((ret = stat(path->u_name, &path->st)) < 0)
+    	path->st_errno = errno;
+   return ret;
+}
+
+/* -------------------------- */
+int of_statdir  (const struct vol *vol, struct path *path)
+{
+static char pathname[ MAXPATHLEN + 1];
+int ret;
+
+    if (*path->m_name) {
+        /* not curdir */
+        return of_stat (path);
+    }
+    path->st_errno = 0;
+    path->st_valid = 1;
+    /* FIXME, what about: we don't have r-x perm anymore ? */
+    strcpy(pathname, "../");
+    strlcat(pathname, path->d_dir->d_u_name, MAXPATHLEN);
+
+    if (!(ret = stat(pathname, &path->st)))
+        return 0;
+        
+    path->st_errno = errno;
+    /* hmm, can't stat curdir anymore */
+    if (errno == EACCES && curdir->d_parent ) {
+       if (movecwd(vol, curdir->d_parent)) 
+           return -1;
+       path->st_errno = 0;
+       if ((ret = stat(path->d_dir->d_u_name, &path->st)) < 0) 
+           path->st_errno = errno;
+    }
+    return ret;
+}
+
+/* -------------------------- */
 struct ofork *
-            of_findname(const char *name, struct stat *st)
+            of_findname(struct path *path)
 {
     struct ofork *of;
     struct file_key key;
-    struct stat buffer;
     
-    if (st == NULL) {
-        st = &buffer;
-        if (stat(name, st) < 0)
-            return NULL;
+    if (!path->st_valid) {
+       of_stat(path);
     }
-    key.dev = st->st_dev;
-    key.inode = st->st_ino;
+    	
+    if (path->st_errno)
+        return NULL;
+
+    key.dev = path->st.st_dev;
+    key.inode = path->st.st_ino;
 
     for (of = ofork_table[hashfn(&key)]; of; of = of->next) {
         if (key.dev == of->key.dev && key.inode == of->key.inode ) {
@@ -325,3 +372,21 @@ struct ofork	*of;
 
     free( of );
 }
+
+/* ----------------------
+
+*/
+struct adouble *of_ad(const struct vol *vol, struct path *path, struct adouble *ad)
+{
+    struct ofork        *of;
+    struct adouble      *adp;
+
+    if ((of = of_findname(path))) {
+        adp = of->of_ad;
+    } else {
+        ad_init(ad, vol->v_adouble);
+        adp = ad;
+    }
+    return adp;
+}
+
