@@ -1,5 +1,5 @@
 /*
- * $Id: uam.c,v 1.22.4.1 2003/06/09 14:30:45 srittau Exp $
+ * $Id: uam.c,v 1.24.6.7.2.2 2004/12/07 18:41:08 bfernhomberg Exp $
  *
  * Copyright (c) 1999 Adrian Sun (asun@zoology.washington.edu)
  * All Rights Reserved.  See COPYRIGHT.
@@ -56,6 +56,12 @@ char *strchr (), *strrchr ();
 #include "auth.h"
 #include "uam_auth.h"
 
+#ifdef AFP3x
+#define utf8_encoding() (afp_version >= 30)
+#else
+#define utf8_encoding() (0)
+#endif
+
 #ifdef TRU64
 #include <netdb.h>
 #include <sia.h>
@@ -64,6 +70,9 @@ char *strchr (), *strrchr ();
 #endif /* TRU64 */
 
 /* --- server uam functions -- */
+#ifndef NO_LOAD_UAM
+extern  int uam_setup(const char *path);
+#endif
 
 /* uam_load. uams must have a uam_setup function. */
 struct uam_mod *uam_load(const char *path, const char *name)
@@ -72,20 +81,23 @@ struct uam_mod *uam_load(const char *path, const char *name)
     struct uam_mod *mod;
     void *module;
 
+#ifndef NO_LOAD_UAM
     if ((module = mod_open(path)) == NULL) {
         LOG(log_error, logtype_afpd, "uam_load(%s): failed to load: %s", name, mod_error());
         return NULL;
     }
+#endif
 
     if ((mod = (struct uam_mod *) malloc(sizeof(struct uam_mod))) == NULL) {
         LOG(log_error, logtype_afpd, "uam_load(%s): malloc failed", name);
         goto uam_load_fail;
     }
 
-    strncpy(buf, name, sizeof(buf));
-    buf[sizeof(buf) - 1] = '\0';
+    strlcpy(buf, name, sizeof(buf));
     if ((p = strchr(buf, '.')))
         *p = '\0';
+
+#ifndef NO_LOAD_UAM
     if ((mod->uam_fcn = mod_symbol(module, buf)) == NULL) {
         LOG(log_error, logtype_afpd, "uam_load(%s): mod_symbol error for symbol %s",
             name,
@@ -106,6 +118,9 @@ struct uam_mod *uam_load(const char *path, const char *name)
         LOG(log_error, logtype_afpd, "uam_load(%s): uam_setup failed", name);
         goto uam_load_err;
     }
+#else
+   uam_setup(name);
+#endif
 
     mod->uam_module = module;
     return mod;
@@ -124,14 +139,83 @@ void uam_unload(struct uam_mod *mod)
 {
     if (mod->uam_fcn->uam_cleanup)
         (*mod->uam_fcn->uam_cleanup)();
+
+#ifndef NO_LOAD_UAM
     mod_close(mod->uam_module);
+#endif    
     free(mod);
 }
 
 /* -- client-side uam functions -- */
-
+#ifndef ATACC
 /* set up stuff for this uam. */
 int uam_register(const int type, const char *path, const char *name, ...)
+{
+    va_list ap;
+    struct uam_obj *uam;
+    int ret;
+
+    if (!name)
+        return -1;
+
+    /* see if it already exists. */
+    if ((uam = auth_uamfind(type, name, strlen(name)))) {
+        if (strcmp(uam->uam_path, path)) {
+            /* it exists, but it's not the same module. */
+            LOG(log_error, logtype_afpd, "uam_register: \"%s\" already loaded by %s",
+                name, path);
+            return -1;
+        }
+        uam->uam_count++;
+        return 0;
+    }
+
+    /* allocate space for uam */
+    if ((uam = calloc(1, sizeof(struct uam_obj))) == NULL)
+        return -1;
+
+    uam->uam_name = name;
+    uam->uam_path = strdup(path);
+    uam->uam_count++;
+
+    va_start(ap, name);
+    switch (type) {
+    case UAM_SERVER_LOGIN_EXT: /* expect four arguments */
+        uam->u.uam_login.login = va_arg(ap, void *);
+        uam->u.uam_login.logincont = va_arg(ap, void *);
+        uam->u.uam_login.logout = va_arg(ap, void *);
+        uam->u.uam_login.login_ext = va_arg(ap, void *);
+        break;
+    
+    case UAM_SERVER_LOGIN: /* expect three arguments */
+        uam->u.uam_login.login_ext = NULL;
+        uam->u.uam_login.login = va_arg(ap, void *);
+        uam->u.uam_login.logincont = va_arg(ap, void *);
+        uam->u.uam_login.logout = va_arg(ap, void *);
+        break;
+    case UAM_SERVER_CHANGEPW: /* one argument */
+        uam->u.uam_changepw = va_arg(ap, void *);
+        break;
+    case UAM_SERVER_PRINTAUTH: /* x arguments */
+    default:
+        break;
+    }
+    va_end(ap);
+
+    /* attach to other uams */
+    ret = auth_register(type, uam);
+    if ( ret) {
+        free(uam->uam_path);
+        free(uam);
+    }
+
+    return ret;
+}
+#endif
+
+#ifdef ATACC
+int uam_register_fn(const int type, const char *path, const char *name, void *fn1, void *fn2, 
+                     void *fn3, void *fn4)
 {
     va_list ap;
     struct uam_obj *uam;
@@ -159,21 +243,26 @@ int uam_register(const int type, const char *path, const char *name, ...)
     uam->uam_path = strdup(path);
     uam->uam_count++;
 
-    va_start(ap, name);
     switch (type) {
+    case UAM_SERVER_LOGIN_EXT: /* expect four arguments */
+        uam->u.uam_login.login_ext = fn4;
+        uam->u.uam_login.login = fn1;
+        uam->u.uam_login.logincont = fn2;
+        uam->u.uam_login.logout = fn3;
+        break;
     case UAM_SERVER_LOGIN: /* expect three arguments */
-        uam->u.uam_login.login = va_arg(ap, void *);
-        uam->u.uam_login.logincont = va_arg(ap, void *);
-        uam->u.uam_login.logout = va_arg(ap, void *);
+        uam->u.uam_login.login_ext = NULL;
+        uam->u.uam_login.login = fn1;
+        uam->u.uam_login.logincont = fn2;
+        uam->u.uam_login.logout = fn3;
         break;
     case UAM_SERVER_CHANGEPW: /* one argument */
-        uam->u.uam_changepw = va_arg(ap, void *);
+        uam->u.uam_changepw = fn1;
         break;
     case UAM_SERVER_PRINTAUTH: /* x arguments */
     default:
         break;
     }
-    va_end(ap);
 
     /* attach to other uams */
     if (auth_register(type, uam) < 0) {
@@ -184,6 +273,7 @@ int uam_register(const int type, const char *path, const char *name, ...)
 
     return 0;
 }
+#endif
 
 void uam_unregister(const int type, const char *name)
 {
@@ -203,31 +293,44 @@ void uam_unregister(const int type, const char *name)
 
 /* --- helper functions for plugin uams --- */
 
-struct passwd *uam_getname(char *name, const int len)
+struct passwd *uam_getname(void *private, char *name, const int len)
 {
+    AFPObj *obj = private;
     struct passwd *pwent;
-    char *user;
-    int i;
+    static char username[256];
+    static char user[256];
+    static char pwname[256];
+    char *p;
+    size_t namelen, gecoslen = 0, pwnamelen = 0;
 
     if ((pwent = getpwnam(name)))
         return pwent;
 
 #ifndef NO_REAL_USER_NAME
-    for (i = 0; i < len; i++)
-        name[i] = tolower(name[i]);
+
+    if ( (size_t) -1 == (namelen = convert_string((utf8_encoding())?CH_UTF8_MAC:obj->options.maccharset,
+				CH_UCS2, name, strlen(name), username, sizeof(username))))
+	return NULL;
 
     setpwent();
     while ((pwent = getpwent())) {
-        if ((user = strchr(pwent->pw_gecos, ',')))
-            *user = '\0';
-        user = pwent->pw_gecos;
+        if ((p = strchr(pwent->pw_gecos, ',')))
+            *p = '\0';
+
+	if ((size_t)-1 == ( gecoslen = convert_string(obj->options.unixcharset, CH_UCS2, 
+				pwent->pw_gecos, strlen(pwent->pw_gecos), user, sizeof(username))) )
+		continue;
+	if ((size_t)-1 == ( pwnamelen = convert_string(obj->options.unixcharset, CH_UCS2, 
+				pwent->pw_name, strlen(pwent->pw_name), pwname, sizeof(username))) )
+		continue;
+
 
         /* check against both the gecos and the name fields. the user
          * might have just used a different capitalization. */
-        if ((strncasecmp(user, name, len) == 0) ||
-                (strncasecmp(pwent->pw_name, name, len) == 0)) {
-            strncpy(name, pwent->pw_name, len);
-            name[len - 1] = '\0';
+
+	if ( (namelen == gecoslen && strncasecmp_w((ucs2_t*)user, (ucs2_t*)username, len) == 0) || 
+		( namelen == pwnamelen && strncasecmp_w ( (ucs2_t*) pwname, (ucs2_t*) username, len) == 0)) {
+            strlcpy(name, pwent->pw_name, len);
             break;
         }
     }
@@ -250,7 +353,6 @@ int uam_checkuser(const struct passwd *pwd)
 		LOG(log_info, logtype_afpd, "uam_checkuser: User %s does not have a shell", pwd->pw_name);
 		return -1;
 	}
-#endif
 
     while ((p = getusershell())) {
         if ( strcmp( p, pwd->pw_shell ) == 0 )
@@ -258,7 +360,6 @@ int uam_checkuser(const struct passwd *pwd)
     }
     endusershell();
 
-#ifndef DISABLE_SHELLCHECK
     if (!p) {
         LOG(log_info, logtype_afpd, "illegal shell %s for %s", pwd->pw_shell, pwd->pw_name);
         return -1;
@@ -268,14 +369,44 @@ int uam_checkuser(const struct passwd *pwd)
     return 0;
 }
 
+int uam_random_string (AFPObj *obj, char *buf, int len)
+{
+    u_int32_t result;
+    int ret;
+    int fd;
+
+    if ( (len <= 0) || (len % sizeof(result)))
+            return -1;
+
+    /* construct a random number */
+    if ((fd = open("/dev/urandom", O_RDONLY)) < 0) {
+        struct timeval tv;
+        struct timezone tz;
+        int i;
+
+        if (gettimeofday(&tv, &tz) < 0)
+            return -1;
+        srandom(tv.tv_sec + (unsigned long) obj + (unsigned long) obj->handle);
+        for (i = 0; i < len; i += sizeof(result)) {
+            result = random();
+            memcpy(buf + i, &result, sizeof(result));
+        }
+    } else {
+        ret = read(fd, buf, len);
+        close(fd);
+        if (ret <= 0)
+            return -1;
+    }
+    return 0;
+}
+
 /* afp-specific functions */
 int uam_afpserver_option(void *private, const int what, void *option,
                          int *len)
 {
 AFPObj *obj = private;
     char **buf = (char **) option; /* most of the options are this */
-    int32_t result;
-    int fd;
+    struct session_info **sinfo = (struct session_info **) option;
 
     if (!obj || !option)
         return -1;
@@ -327,29 +458,10 @@ AFPObj *obj = private;
         break;
 
     case UAM_OPTION_RANDNUM: /* returns a random number in 4-byte units. */
-        if (!len || (*len < 0) || (*len % sizeof(result)))
+        if (!len)
             return -1;
 
-        /* construct a random number */
-        if ((fd = open("/dev/urandom", O_RDONLY)) < 0) {
-            struct timeval tv;
-            struct timezone tz;
-            char *randnum = (char *) option;
-            int i;
-
-            if (gettimeofday(&tv, &tz) < 0)
-                return -1;
-            srandom(tv.tv_sec + (unsigned long) obj + (unsigned long) obj->handle);
-            for (i = 0; i < *len; i += sizeof(result)) {
-                result = random();
-                memcpy(randnum + i, &result, sizeof(result));
-            }
-        } else {
-            result = read(fd, option, *len);
-            close(fd);
-            if (result < 0)
-                return -1;
-        }
+        return uam_random_string(obj, option, *len);
         break;
 
     case UAM_OPTION_HOSTNAME:
@@ -383,9 +495,30 @@ AFPObj *obj = private;
         *buf = (void *) &obj->uam_cookie;
         break;
     case UAM_OPTION_KRB5SERVICE:
-        *buf = obj->options.k5service;
+	*buf = obj->options.k5service;
         if (len)
-            *len = strlen(obj->options.k5service);
+            *len = (*buf)?strlen(*buf):0;
+	break;
+    case UAM_OPTION_KRB5REALM:
+	*buf = obj->options.k5realm;
+        if (len)
+            *len = (*buf)?strlen(*buf):0;
+	break;
+    case UAM_OPTION_FQDN:
+	*buf = obj->options.fqdn;
+        if (len)
+            *len = (*buf)?strlen(*buf):0;
+	break;
+    case UAM_OPTION_MACCHARSET:
+        *((int *) option) = obj->options.maccharset;
+        *len = sizeof(obj->options.maccharset);
+        break;
+    case UAM_OPTION_UNIXCHARSET:
+        *((int *) option) = obj->options.unixcharset;
+        *len = sizeof(obj->options.unixcharset);
+        break;
+    case UAM_OPTION_SESSIONINFO:
+        *sinfo = &(obj->sinfo);
         break;
     default:
         return -1;
